/*
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999-2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Struts", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */
package org.apache.struts.webapp.example.oracle;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.util.ArrayList;
import javax.sql.DataSource;
import org.apache.struts.action.Action;
import org.apache.struts.action.Tcr;
import org.apache.struts.action.TcrAction;
import org.apache.struts.action.TcrForm;

/**
 * Fill out the lists of column names and rows for the selected database table.
 *
 * @author Alec Missine
 */
public class SelectTable extends TcrAction {

	protected Tcr newTcr( TcrForm form )
	{
		return new TimeConsumingRequest( form );
	}

	static class TimeConsumingRequest extends Tcr {

		ArrayList columns = new ArrayList(), data = new ArrayList();
		String schema, table, whereClause;

		TimeConsumingRequest( TcrForm form ) { 
			super( form );
			TableForm tf = (TableForm)form;
			schema = ((SchemaForm)tf.session.getAttribute( "schemaForm" )).
				getSchema();
			table = tf.getTable();
			whereClause = tf.getWhereClause();
			whereClause = whereClause == null ? "" : " where " + whereClause;
			whereClause = this.whereClause.replace( ':', '=' );
		}

		public void run() 
		{
//long t0 = System.currentTimeMillis(), t1 = 0;
			try { 
//Thread.sleep( 5000 );
				Connection connection = ((DataSource)form.getServlet().
					getServletContext().getAttribute( Action.DATA_SOURCE_KEY )).
					getConnection();
				DatabaseMetaData dbmd = connection.getMetaData();
				ResultSet rs = dbmd.getColumns( null, schema, table, "%" );
				while( rs.next() ) {	//	1st row, column names: -------
					columns.add( new ColumnName( rs.getString( 4 ) ) );
				}
				rs.close();
				int iColumns = columns.size();
				String sql = "select * from " + 
					schema + '.' + table + whereClause;
//System.out.println( getClass().getName() + ".run: sql = \"" + sql + "\"" );
				rs = connection.createStatement().executeQuery( sql );
				while( rs.next() ) {	//	the table's data: ------------
					Row row = new Row( iColumns );
					for( int i = 0; i < iColumns; i++ )
						row.add( new Field( rs.getObject( i + 1 ) ) );
					data.add( row );
				}
				done();
//System.out.println( getClass().getName() + ".run: completed in " + ((t1 = System.currentTimeMillis()) - t0) + " ms" );
				rs.close();
				connection.close();
			} catch( Exception e ) { e.printStackTrace(); }
//System.out.println( getClass().getName() + ".run: closed in " + (System.currentTimeMillis() - t1) + " ms" );
		}

		protected void copyTo( TcrForm form ) 
		{
			((TableForm)form).setColumns( columns );
			((TableForm)form).setData( data );
//System.out.println( getClass().getName() + ".copyTo: table = \"" + table + "\"" );
			((TableForm)form).setTable( table );
		}
	}

	public static class Field {

		Object value;
		Field( Object value ) { this.value = value; }
		public String getValue() { 
			return value == null? "" : value.toString(); 
		}
	}

	public static class Row {

		ArrayList fields;
		Row( int i ) { fields = new ArrayList( i ); }
		public ArrayList getFields() { return fields; }
		public void add( Object field ) { fields.add( field ); }
	}

	public static class ColumnName {

		String name;
		ColumnName( String name ) { this.name = name; }
		public String getName() { return name; }
	}
}
